package org.dbms.service;

import org.dbms.dto.CreateOrderDTO;
import org.dbms.models.Client;
import org.dbms.models.Driver;
import org.dbms.models.Order;
import org.dbms.storageContracts.IOrderStorage;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.RequestParam;

import java.time.LocalDate;
import java.util.Date;
import java.util.List;

@Service
public class OrderService {
    private IOrderStorage orderStorage;
    private ClientService clientService;
    private DriverService driverService;

    public OrderService(IOrderStorage orderStorage, ClientService clientService, DriverService driverService) {
        this.orderStorage = orderStorage;
        this.clientService = clientService;
        this.driverService = driverService;
    }

    @Transactional
    public List<Order> getOrdersByClient(String login) {
        Client client = clientService.getClientByLogin(login);
        return orderStorage.getOrdersByClientId(client.getId());
    }

    @Transactional
    public List<Order> getOrdersPageByClient(String login, int size, int page) {
        Client client = clientService.getClientByLogin(login);
        return orderStorage.readPageByClientId(size, page, client.getId());
    }

    @Transactional
    public void createOrder(CreateOrderDTO orderDTO) {
        Client client = clientService.getClientByLogin(orderDTO.getLogin());
        Driver driver = driverService.getDriverById(orderDTO.getDriverId());
        Order order = new Order(orderDTO.getFrom(), orderDTO.getTo(), calculatePrice(orderDTO),
                                orderDTO.getDeliveryWeight(), driver, client);

        orderStorage.insert(order);
    }
    @Transactional
    public int getPagesCount(String login, int size) {
        Client client = clientService.getClientByLogin(login);
        int count = orderStorage.getCountByClientId(client.getId());
        int res = count / size;
        if(count % size != 0) res++;
        return res;
    }
    @Transactional
    private double calculatePrice(CreateOrderDTO orderDTO) {
        Driver driver = driverService.getDriverById(orderDTO.getDriverId());
        double frigileCoef = 1.0;
        if(orderDTO.isFragile()) frigileCoef = 1.1;

        double driverCoef = LocalDate.now().getYear() - driver.getLicenseYear().getYear() - 1900;
        driverCoef += 5;
        driverCoef /= 5;

        double res = orderDTO.getDeliveryWeight() * 50 * frigileCoef * driverCoef;
        return res;
    }
}
